// --------------------------------------------------------------------------------------
//
// FishRobo.CS                                                     Ulrich Mller 07.05.05
//                                                                 PCS1700       C# Final
// Klasse fr den Umgang mit fischertechnik Interfaces
// Deklarationen fr umFish40.DLL und mehr
//
// --------------------------------------------------------------------------------------

using System;
using System.Runtime.InteropServices;

namespace HelloCSROBO
{
	public class FishFaceException: ApplicationException {
		public FishFaceException() {}
		public FishFaceException(string message) : base(message) {}
		public FishFaceException(string message, Exception inner) : base(message, inner){}
	}
	public class FishRobo
	{
		// --- umFish40.DLL Deklarationen ----------------------------------------
		[DllImport("umFish40.DLL")]
		private static extern int rbOpenInterfaceUSB(int ifTyp, int SerialNr);
		[DllImport("umFish40.DLL")]
		private static extern int rbOpenInterfaceCOM(int ifTyp, 
			                                         int ComNr, int AnalogZyklen);
		[DllImport("umFish40.DLL")]
		private static extern int rbCloseInterface(int iHandle);
		[DllImport("umFish40.DLL")]
		private static extern int rbGetInput(int iHandle, int InputNr);
		[DllImport("umFish40.DLL")]
		private static extern int rbGetAnalog(int iHandle, int AnalogNr);
		[DllImport("umFish40.DLL")]
		private static extern int rbGetVoltage(int iHandle, int VoltNr);
		[DllImport("umFish40.DLL")]
		private static extern int rbSetMotor(int iHandle, int MotNr, int Dir);
		[DllImport("umFish40.DLL")]
		private static extern int rbSetMotorEx(int iHandle, 
			                                   int MotNr, int Dir, int Speed);
		[DllImport("umFish40.DLL")]
		private static extern int rbSetLamp(int iHandle, int LampNr, int OnOff);
		[DllImport("umFish40.DLL")]
		private static extern int rbSetLampEx(int iHandle, 
			                                  int LampNr, int OnOff, int Power);
		[DllImport("umFish40.DLL")]
		private static extern int rbRobMotor(int iHandle, 
			                                 int MotNr, int Dir, int Speed, int ICount);
		[DllImport("umFish40.DLL")]
		private static extern int rbGetCounter(int iHandle, int CounterNr);
		[DllImport("umFish40.DLL")]
		private static extern int rbSetCounter(int iHandle, int CounterNr, int ICount);
		[DllImport("umFish40.DLL")]
		private static extern int rbClearCounter(int iHandle);

		// --- private (globale) Variable ----------------------------------------

		private int iHandle;

		public const uint rbFehler = 0xE0000001;

		// --- FishRobo : Constructor ---------------------------------------------

		public FishRobo() {}
		
		public void OpenInterface(int ifTyp, int SerialNr) {
			iHandle = rbOpenInterfaceUSB(ifTyp, SerialNr);
			if((uint)iHandle == rbFehler)
				throw(new FishFaceException("InterfaceProblem.Open"));
		}
		public void OpenInterface(int ifTyp, int ComNr, int AnalogZyklen) {
			iHandle = rbOpenInterfaceCOM(ifTyp, ComNr, AnalogZyklen);
			if((uint)iHandle == rbFehler)
				throw(new FishFaceException("InterfaceProblem.Open"));
		}
		public void CloseInterface() {
			rbCloseInterface(iHandle);
		}
		public bool GetInput(int InputNr) {
			int res = rbGetInput(iHandle, InputNr);
			if((uint)res == rbFehler)
				throw(new FishFaceException("InterfaceProblem.GetInput"));
			return (res == 0) ? false : true;
		}
		public int GetAnalog(int AnalogNr) {
			int res = rbGetAnalog(iHandle, AnalogNr);
			if((uint)res == rbFehler) 
				throw(new FishFaceException("InterfaceProblem.GetAnalog"));
			return res;
		}
		public int GetVoltage(int VoltNr) {
			int res = rbGetVoltage(iHandle, VoltNr);
			if((uint)res == rbFehler) 
				throw(new FishFaceException("InterfaceProblem.GetVoltage"));
			return res;
		}
		public void SetMotor(int MotNr, int Dir) {
			if((uint)rbSetMotor(iHandle, MotNr, Dir) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.SetMotor"));
		}
		public void SetMotor(int MotNr, int Dir, int Speed) {
			if((uint)rbSetMotorEx(iHandle, MotNr, Dir, Speed) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.SetMotor"));
		}
		public void SetLamp(int LampNr, int OnOff) {
			if((uint)rbSetLamp(iHandle, LampNr, OnOff) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.SetLamp"));
		}
		public void SetLamp(int LampNr, int OnOff, int Power) {
			if((uint)rbSetLampEx(iHandle, LampNr, OnOff, Power) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.SetLamp"));
		}
		public void SetMotor(int MotNr, int Dir, int Speed, int Counter) {
			if((uint)rbRobMotor(iHandle, MotNr, Dir, Speed, Counter) == rbFehler)
				throw(new FishFaceException("InterfaceProblem.SetMotor"));
		}
		public int GetCounter(int CounterNr) {
			int res = rbGetCounter(iHandle, CounterNr);
			if((uint)res == rbFehler)
				throw(new FishFaceException("InterfaceProblem.GetCounter"));
			return res;
		}
		public void SetCounter(int CounterNr, int ICount) {
			if((uint)rbSetCounter(iHandle, CounterNr, ICount) == rbFehler)
				throw(new FishFaceException("InterfaceProbleim.SetCounter"));
		}
		public void ClearCounter() {
			if((uint)rbClearCounter(iHandle) == rbFehler)
				throw(new FishFaceException("InterfaceProbleim.ClearCounter"));
		}
	}
}
