// -----------------------------------------------------------------------------
//
// umFish40.CPP                                           Ulrich Mller 23.04.06
//                                                        PCS1700       VC++ 6.0
// Basisfunktionen fr das fischertechnik Interfaces unter Nutzung
//                 der FtLib_Static-LIBCMT_...lib
// Der Funktionsumfang orientiert sich an umFish30.DLL - cs-Variante
// 14.11.05 : Anpassung an FtLib v0.49
// 09.03.06 : Anpassung an FiLib v0.56 weiterhin alte umFtLib.h
//            keine neuen Funktionen. Luft mit allen ROBO's mit verschiedenen
//            Firmwarestnden, auerdem mit dem Intelligent Interface
// 19.3.06 :  Version 4.1 Mit Communication zwischen RF Datalink und zugeordnetem
//            ROBO Interface RF 2/1 und weiteren RF 2/2 .... neue FtLib
// 23.4.06 :  Anpassungen an FtLib v0.59
//
// Fehler-Return allgemein rbFehler, OK = 0 bzw. def. Rckgabewert < 0xE0000000
//
// ------------------------------------------------------------------------------

#include <Windows.h>
#include "FtLib.h"
#include "umFish40.H"

// ----------- globale Konstanten ----------------------------------------------

const int rbFehler = 0xE0000001L;

// ----------- globale Variable ------------------------------------------------

RoboInstanz rbI[8];
const UINT  rfMax = 150;
UINT        rfAnz;
UINT        rfIn;
UINT        rfOut;
SMESSAGE    rfMsg[rfMax];

//
// ----------- interne Funktionen ---------------------------------------------
//

// CCounter -------------------------------------------------------------------
//
// Zentrale CallBack-Routine der TransferArea
//
// -----------------------------------------------------------------------------

void CCounter(void* aktRoboInstanz) {
	DWORD iMot, iEnd, iImpuls, MotMode, MotDir;
	DWORD iStatusNeu, iStatusDelta;
	BOOL  MotEnd;
	DWORD c, n;
	RoboInstanz* rbI = (RoboInstanz*)aktRoboInstanz;

	if(rbI->ftDCB == NULL) return;
	
	iStatusNeu  = rbI->ftDCB->E_Main;
	iStatusNeu |= rbI->ftDCB->E_Sub1 << 8;
	iStatusNeu |= rbI->ftDCB->E_Sub2 << 16;
	iStatusNeu |= rbI->ftDCB->E_Sub3 << 24;
	iStatusDelta = rbI->iStatusAlt ^ iStatusNeu;
	rbI->iStatusAlt   = iStatusNeu;

	for(iMot = 0; iMot < 16; iMot++) {
		iEnd    = iMot * 2;
		iImpuls = iMot * 2 + 1;
		MotMode = (rbI->mModeStatus & MMaske[iMot]) ? 1 : 0;
		switch(MotMode) {
		case 0:  // --- NormalMode
			if((iStatusDelta & EMaske[iEnd])    > 0) rbI->iCounter[iEnd]++;
			if((iStatusDelta & EMaske[iImpuls]) > 0) rbI->iCounter[iImpuls]++;
			break;
		case 1:  // --- RobMode
			c = iMot >> 2;
			n = iMot % 4;
			switch(c) {
				case 0:
					if((iStatusDelta & EMaske[iImpuls]) > 0) rbI->iCounter[iImpuls]--;
					MotDir = (rbI->ftDCB->M_Main & ~MAus[n]) >> n * 2;
					MotEnd = (rbI->ftDCB->E_Main & EMaske[n*2]) ? true : false;
					if((rbI->iCounter[iImpuls] == 0) || ((MotDir == ftiLinks) && MotEnd)) {
						rbI->ftDCB->M_Main &= MAus[n];
						rbI->mModeStatus &= ~MMaske[iMot];
						rbI->iCounter[iImpuls] = 0;
					}
					break;
				case 1:
					if((iStatusDelta & EMaske[iImpuls]) > 0) rbI->iCounter[iImpuls]--;
					MotDir = (rbI->ftDCB->M_Sub1 & ~MAus[n]) >> n * 2;
					MotEnd = (rbI->ftDCB->E_Sub1 & EMaske[n*2]) ? true : false;
					if((rbI->iCounter[iImpuls] == 0) || ((MotDir == ftiLinks) && MotEnd)) {
						rbI->ftDCB->M_Sub1 &= MAus[n];
						rbI->mModeStatus &= ~MMaske[iMot];
						rbI->iCounter[iImpuls] = 0;
					}
					break;
				case 2:
					if((iStatusDelta & EMaske[iImpuls]) > 0) rbI->iCounter[iImpuls]--;
					MotDir = (rbI->ftDCB->M_Sub2 & ~MAus[n]) >> n * 2;
					MotEnd = (rbI->ftDCB->E_Sub2 & EMaske[n*2]) ? true : false;
					if((rbI->iCounter[iImpuls] == 0) || ((MotDir == ftiLinks) && MotEnd)) {
						rbI->ftDCB->M_Sub2 &= MAus[n];
						rbI->mModeStatus &= ~MMaske[iMot];
						rbI->iCounter[iImpuls] = 0;
					}
					break;
				case 3:
					if((iStatusDelta & EMaske[iImpuls]) > 0) rbI->iCounter[iImpuls]--;
					MotDir = (rbI->ftDCB->M_Sub3 & ~MAus[n]) >> n * 2;
					MotEnd = (rbI->ftDCB->E_Sub3 & EMaske[n*2]) ? true : false;
					if((rbI->iCounter[iImpuls] == 0) || ((MotDir == ftiLinks) && MotEnd)) {
						rbI->ftDCB->M_Sub3 &= MAus[n];
						rbI->mModeStatus &= ~MMaske[iMot];
						rbI->iCounter[iImpuls] = 0;
					}
					break;
			}
		}
	}
}

// SetMotorAll -----------------------------------------------------------------
//
// Schalten eines Motors
//
// -----------------------------------------------------------------------------

int SetMotorAll(int iHandle, int Ext, int n, int Dir, int Speed) {
	switch(Ext) {
		case 0:         // --- Main ----------
			rbI[iHandle].ftDCB->M_Main &= MAus[n];
			switch (Dir) {
				case ftiAus:  break;	
				case ftiLinks: {
					rbI[iHandle].ftDCB->M_Main |= MLinks[n]; 
					rbI[iHandle].ftDCB->MPWM_Main[n*2] = Speed;
					break; }
				case ftiRechts:  {
					rbI[iHandle].ftDCB->M_Main |= MRechts[n]; 
					rbI[iHandle].ftDCB->MPWM_Main[n*2+1] = Speed;
					break;}
			}
			break;
		case 1:        // --- Extension 1 ------
			rbI[iHandle].ftDCB->M_Sub1 &= MAus[n];
			switch (Dir) {
				case ftiAus:  break;	
				case ftiLinks: {
					rbI[iHandle].ftDCB->M_Sub1 |= MLinks[n]; 
					rbI[iHandle].ftDCB->MPWM_Sub1[n*2] = Speed;
					break; }
				case ftiRechts:  {
					rbI[iHandle].ftDCB->M_Sub1 |= MRechts[n]; 
					rbI[iHandle].ftDCB->MPWM_Sub1[n*2+1] = Speed;
					break;}
			}
			break;
		case 2:        // --- Extension 2 ------
			rbI[iHandle].ftDCB->M_Sub2 &= MAus[n];
			switch (Dir) {
				case ftiAus:  break;	
				case ftiLinks: {
					rbI[iHandle].ftDCB->M_Sub2 |= MLinks[n]; 
					rbI[iHandle].ftDCB->MPWM_Sub2[n*2] = Speed;
					break; }
				case ftiRechts:  {
					rbI[iHandle].ftDCB->M_Sub2 |= MRechts[n]; 
					rbI[iHandle].ftDCB->MPWM_Sub2[n*2+1] = Speed;
					break;}
			}
			break;
		case 3:        // --- Extension 3 ------
			rbI[iHandle].ftDCB->M_Sub3 &= MAus[n];
			switch (Dir) {
				case ftiAus:  break;	
				case ftiLinks: {
					rbI[iHandle].ftDCB->M_Sub3 |= MLinks[n]; 
					rbI[iHandle].ftDCB->MPWM_Sub3[n*2] = Speed;
					break; }
				case ftiRechts:  {
					rbI[iHandle].ftDCB->M_Sub3 |= MRechts[n]; 
					rbI[iHandle].ftDCB->MPWM_Sub3[n*2+1] = Speed;
					break;}
			}
			break;
		default : return rbFehler;
	}
	return 0;
}

// MessagesIn ------------------------------------------------------------------
//
// Puffern der ber Funk einkommenden Messages bei FT_ROBO_RF_DATA_LINK
//
// -----------------------------------------------------------------------------

void MessagesIn(SMESSAGE* inM) {
	if(rfAnz < rfMax) {
		rfMsg[rfIn].W.ucHwId  = inM->W.ucHwId;
		rfMsg[rfIn].W.ucSubId = inM->W.ucSubId;
		rfMsg[rfIn].W.uiMsg   = inM->W.uiMsg;
		rfMsg[rfIn].W.uiMsgId = inM->W.uiMsgId;
		rfIn = rfIn < rfMax ? rfIn+1 : 0;
		rfAnz++;
	}
}

// OpenInterfaceAll ------------------------------------------------------------
//
// Setzen Ausgangswerte, Starten TransferArea
//
// ------------------------------------------------------------------------------

int OpenInterfaceAll(FT_HANDLE ftHandle) {
	int i;
	// --- Suchen eines freien RoboInstanz-Eintrage in rbI ----
  int iHandle = rbFehler;
	for(i = 0; i < 8; i++)
		if(rbI[i].ftHandle == 0) {
			iHandle = i; break; }
		if(iHandle == rbFehler) return rbFehler;

	rbI[iHandle].ftHandle = ftHandle;

	// --- Aufsetzen TransferArea ----------------
	rbI[iHandle].ftEvents.Context              = &rbI[iHandle];
	rbI[iHandle].ftEvents.hEvent               = NULL;
	rbI[iHandle].ftEvents.Message              = NULL;
	rbI[iHandle].ftEvents.NotificationCallback = CCounter;
	rbI[iHandle].ftEvents.CallbackMessage      = NULL;
	rbI[iHandle].ftEvents.WindowsHandle        = NULL;
	rbI[iHandle].ftDCB                         = NULL;
	if(GetFtDeviceTyp(ftHandle) == FT_ROBO_RF_DATA_LINK) {
		rfAnz = 0;
		rfIn  = 0;
		rfOut = 0;
		rbI[iHandle].ftEvents.CallbackMessage    = MessagesIn;
		if(StartFtTransferAreaWithCommunication(rbI[iHandle].ftHandle, 
			&rbI[iHandle].ftEvents)) return rbFehler;
	}
	else {
		if(StartFtTransferArea(rbI[iHandle].ftHandle, &rbI[iHandle].ftEvents)) 
			return rbFehler;
	}
	rbI[iHandle].ftDCB = GetFtTransferAreaAddress(rbI[iHandle].ftHandle);
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;

	// --- weiteres Besetzen RoboInstanz ------
	for(i = 0; i < 32; i++) rbI[iHandle].iCounter[i] = 0;
	rbI[iHandle].mModeStatus = 0;
	return iHandle;
}

//
// ------------ public Funktionen ----------------------------------------------
//

// rbOpenInterfaceUSB ----------------------------------------------------------
//
// ffnen eines Interface mit Anhang ber einen USB-Anschlu
//
// -----------------------------------------------------------------------------

int __stdcall rbOpenInterfaceUSB(int ifTyp, int SerialNr) {
	FT_HANDLE usbHandle;
	
	if (IsFtLibInit() == FTLIB_ERR_LIB_IS_NOT_INITIALIZED) {
		if(InitFtLib()) return rbFehler;
	}
	DWORD res = InitFtUsbDeviceList();
	if(!(res == 0 || res == FTLIB_ERR_SOME_DEVICES_ARE_OPEN)) return rbFehler;
	UINT anzUSBDev = GetNumFtUsbDevice();
	if(!anzUSBDev) return rbFehler;
	if(ifTyp == 0) usbHandle = GetFtUsbDeviceHandle(0);
	else usbHandle = GetFtUsbDeviceHandleSerialNr(SerialNr, ifTyp);
	if(!usbHandle) return rbFehler;
	if(OpenFtUsbDevice(usbHandle)) return rbFehler;
	return OpenInterfaceAll(usbHandle);
}

// rbOpenInterfaceRF ----------------------------------------------------------
//
// ffnen eines Interface mit Anhang ber ein ROBO RF-Datalink
//
// -----------------------------------------------------------------------------

int __stdcall rbOpenInterfaceRF(int SerialNrInterface) {
	FT_HANDLE  usbHandle;
	bool Found;
	
	if (IsFtLibInit() == FTLIB_ERR_LIB_IS_NOT_INITIALIZED) {
		if(InitFtLib()) return rbFehler;
	}
	DWORD res = InitFtUsbDeviceList();
	if(!(res == 0 || res == FTLIB_ERR_SOME_DEVICES_ARE_OPEN)) return rbFehler;
	UINT anzUSBDev = GetNumFtUsbDevice();
	if(!anzUSBDev) return rbFehler;
	Found = false;
	for(UINT i = 0; i < anzUSBDev; i++) {
		usbHandle = GetFtUsbDeviceHandle(i);
		if(GetFtSerialNr(usbHandle) == (UINT)SerialNrInterface) {
			Found = true;
			break;}
	}
	if(!Found) return rbFehler;
	if(OpenFtUsbDevice(usbHandle)) return rbFehler;
	return OpenInterfaceAll(usbHandle);
}

// rbOpenInterfaceCOM ----------------------------------------------------------
//
// ffnen eines Interface mit Anhang ber einen COM-Anschlu
//
// -----------------------------------------------------------------------------

int __stdcall rbOpenInterfaceCOM(int ifTyp, int ComNr, int AnalogZyklen) {
	DWORD Error;
	DWORD* pError = &Error;

	if (IsFtLibInit() == FTLIB_ERR_LIB_IS_NOT_INITIALIZED) {
		if(InitFtLib()) return rbFehler;
	}
	FT_HANDLE comHandle = OpenFtCommDevice(ComNr, ifTyp, AnalogZyklen, pError);
	if(*pError) return rbFehler;
	return OpenInterfaceAll(comHandle);
}

// rbCloseInterface -------------------------------------------------------------
//
// Stop TransferArea, Schlieen Schnittstelle
//
// ------------------------------------------------------------------------------

int __stdcall rbCloseInterface(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(StopFtTransferArea(rbI[iHandle].ftHandle)) return rbFehler;
	if(CloseFtDevice(rbI[iHandle].ftHandle)) return rbFehler;
	rbI[iHandle].ftDCB = NULL;
	rbI[iHandle].ftHandle = 0;
	return 0;
}

// rbGetInput -------------------------------------------------------------------
//
// Abfrage eines I-Einganges
//
// ------------------------------------------------------------------------------

int __stdcall rbGetInput(int iHandle, int InputNr) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	InputNr--;
	DWORD c = InputNr >> 3;
	DWORD n = InputNr % 8;
	switch(c) {
		case 0:  return (rbI[iHandle].ftDCB->E_Main & EMaske[n]) ? 1 : 0; break;
		case 1:	 return (rbI[iHandle].ftDCB->E_Sub1 & EMaske[n]) ? 1 : 0; break;
		case 2:	 return (rbI[iHandle].ftDCB->E_Sub2 & EMaske[n]) ? 1 : 0; break;
		case 3:	 return (rbI[iHandle].ftDCB->E_Sub3 & EMaske[n]) ? 1 : 0; break;
		default: return rbFehler;
	}
}

// rbGetInputs ------------------------------------------------------------------
//
// Status aller I-Eingnge
//
// ------------------------------------------------------------------------------

int __stdcall rbGetInputs(int iHandle) {
	DWORD InputStatus = 0;
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	InputStatus  = rbI[iHandle].ftDCB->E_Main;
	InputStatus |= rbI[iHandle].ftDCB->E_Sub1 << 8;
	InputStatus |= rbI[iHandle].ftDCB->E_Sub2 << 16;
	InputStatus |= rbI[iHandle].ftDCB->E_Sub3 << 24;
	return InputStatus;
}

// rbGetAnalog ------------------------------------------------------------------
//
// Abfrage eines AX / AY-Einganges
//
// ------------------------------------------------------------------------------

int __stdcall rbGetAnalog(int iHandle, int AnalogNr) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	switch (AnalogNr) {
		case ftiAX:    return rbI[iHandle].ftDCB->AX;   break;
		case ftiAY:    return rbI[iHandle].ftDCB->AY;   break;
		case ftiAXS1:  return rbI[iHandle].ftDCB->AXS1; break;
		case ftiAXS2:  return rbI[iHandle].ftDCB->AXS2; break;
		case ftiAXS3:  return rbI[iHandle].ftDCB->AXS3; break;
		default:       return rbFehler;
	}
}

// rbGetIRKey -------------------------------------------------------------------
//
// Abfrage eines IR (Infrarot)Keys des IR Control Sets
//
// ------------------------------------------------------------------------------

int __stdcall rbGetIRKey(int iHandle, int Code, int KeyNr) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	KeyNr--;
	switch(Code) {
		case 0:    // --- Unabhngig vom Code
			return rbI[iHandle].ftDCB->IRKeys16[KeyNr];
			break;
		case 1:    // --- Empfnger 1
			return rbI[iHandle].ftDCB->IRKeys16Code1[KeyNr];
			break;
		case 2:    // --- Empfnger 2
			return rbI[iHandle].ftDCB->IRKeys16Code2[KeyNr];
			break;
		default:
			return rbFehler;
	}
}

// rbGetVoltage -----------------------------------------------------------------
//
// Abfrage eines A1 / A2 / AV-Einganges
//
// ------------------------------------------------------------------------------

int __stdcall rbGetVoltage(int iHandle, int VoltNr) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	switch (VoltNr) {
		case ftiA1:  return rbI[iHandle].ftDCB->A1; break;
		case ftiA2:  return rbI[iHandle].ftDCB->A2; break;
		case ftiAV:  return rbI[iHandle].ftDCB->AV; break;
		default:     return 0;
	}
}

// rbSetMotorEx -----------------------------------------------------------------
//
// Setzen eines M-Ausgangen
//
// ------------------------------------------------------------------------------

int __stdcall rbSetMotorEx(int iHandle, int MotNr, int Dir, int Speed) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	MotNr--;
	rbI[iHandle].mModeStatus   &= MAus[MotNr];
	return SetMotorAll(iHandle, MotNr >> 2, MotNr % 4, Dir, Speed);
}

int __stdcall rbSetMotor(int iHandle, int MotNr, int Dir) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	return rbSetMotorEx(iHandle, MotNr, Dir, 7);
}

// rbGetMotors ----------------------------------------------------------------
//
// (Direction) Status aller M-Ausgnge
//
// ----------------------------------------------------------------------------

int __stdcall rbGetMotors(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	DWORD MotorStatus = 0;
	MotorStatus  = rbI[iHandle].ftDCB->M_Main;
	MotorStatus |= rbI[iHandle].ftDCB->M_Sub1 << 8;
	MotorStatus |= rbI[iHandle].ftDCB->M_Sub2 << 16;
	MotorStatus |= rbI[iHandle].ftDCB->M_Sub3 << 24;
	return MotorStatus;
}

// rbSetMotors ----------------------------------------------------------------
//
// Setzen des (Direction) Status aller M-Ausgnge, Speed = 7, Mode = 0
//
// ----------------------------------------------------------------------------

int __stdcall rbSetMotors(int iHandle, int MotorStatus) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	rbI[iHandle].ftDCB->M_Main =  MotorStatus & 0x000000FF;
	rbI[iHandle].ftDCB->M_Sub1 = (MotorStatus & 0x0000FF00) >> 8;
	rbI[iHandle].ftDCB->M_Sub2 = (MotorStatus & 0x00FF0000) >> 16;
	rbI[iHandle].ftDCB->M_Sub3 = (MotorStatus & 0xFF000000) >> 24;
	rbI[iHandle].mModeStatus   = 0;
	for(int i = 0; i < 8; i++) {
		rbI[iHandle].ftDCB->MPWM_Main[i] = 7;
		rbI[iHandle].ftDCB->MPWM_Sub1[i] = 7;
		rbI[iHandle].ftDCB->MPWM_Sub2[i] = 7;
		rbI[iHandle].ftDCB->MPWM_Sub3[i] = 7;
	}
	rbI[iHandle].mModeStatus = 0;
	return 0;
}

// rbSetMotorsEx --------------------------------------------------------------
//
// Setzen des (Direction) Status, Speed aller M-Ausgnge, Mode = 0
//
// ----------------------------------------------------------------------------

int __stdcall rbSetMotorsEx(int iHandle, int MotorStatus, 
														int SpeedStatus, int SpeedStatus16) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	rbI[iHandle].ftDCB->M_Main =  MotorStatus & 0x000000FF;
	rbI[iHandle].ftDCB->M_Sub1 = (MotorStatus & 0x0000FF00) >> 8;
	rbI[iHandle].ftDCB->M_Sub2 = (MotorStatus & 0x00FF0000) >> 16;
	rbI[iHandle].ftDCB->M_Sub3 = (MotorStatus & 0xFF000000) >> 24;
	rbI[iHandle].mModeStatus   = 0;
	int i2, i21;
	char Speed;
	for(int i = 0; i < 4; i++) {
		i2  = i*2;
		i21 = i2+1;
		Speed = (SpeedStatus   & (0xF << (i*4))) >> (i*4);
		rbI[iHandle].ftDCB->MPWM_Main[i2]  =  Speed;
		rbI[iHandle].ftDCB->MPWM_Main[i21] =  Speed;
		Speed = (SpeedStatus   & (0xF << (i*4+16))) >> (i*4+16);
		rbI[iHandle].ftDCB->MPWM_Sub1[i2]  = Speed;
		rbI[iHandle].ftDCB->MPWM_Sub1[i21] = Speed;
		Speed = (SpeedStatus16 & (0xF << (i*4))) >> (i*4);
		rbI[iHandle].ftDCB->MPWM_Sub2[i2]  = Speed;
		rbI[iHandle].ftDCB->MPWM_Sub2[i21] = Speed;
		Speed = (SpeedStatus16 & (0xF << (i*4+16))) >> (i*4+16);
		rbI[iHandle].ftDCB->MPWM_Sub3[i2]  = Speed;
		rbI[iHandle].ftDCB->MPWM_Sub3[i21] = Speed;
	}
	rbI[iHandle].mModeStatus = 0;
	return 0;
}

// rbGetModeStatus ------------------------------------------------------------
//
// Abfrage des ModeStatus eines M-Ausganges
//
// ----------------------------------------------------------------------------

int __stdcall rbGetModeStatus(int iHandle, int MotNr) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	MotNr--;
	return (rbI[iHandle].mModeStatus & MMaske[MotNr]) >> (MotNr*2);
}

// rbSetModeStatus ------------------------------------------------------------
//
// Setzen des ModeStatus eines M-Ausganges
//
// ----------------------------------------------------------------------------

int __stdcall rbSetModeStatus(int iHandle, int MotNr, int Mode) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	MotNr--;
	rbI[iHandle].mModeStatus &= ~MMaske[MotNr];
	rbI[iHandle].mModeStatus |= Mode << (MotNr * 2);
	return 0;
}

// rbSetLampEx ----------------------------------------------------------------
//
// Setzen eines O-Ausganges
//
// ----------------------------------------------------------------------------

int __stdcall rbSetLampEx(int iHandle, int LampNr, int OnOff, int Power) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	LampNr--;
	DWORD c = LampNr >> 3;
	DWORD n = LampNr % 8;
	switch(c) {
		case 0:
			if(OnOff) rbI[iHandle].ftDCB->M_Main |=  EMaske[n];
			else      rbI[iHandle].ftDCB->M_Main &= ~EMaske[n];
			rbI[iHandle].ftDCB->MPWM_Main[n] = Power;
			break;
		case 1:
			if(OnOff) rbI[iHandle].ftDCB->M_Sub1 |=  EMaske[n];
			else      rbI[iHandle].ftDCB->M_Sub1 &= ~EMaske[n];
			rbI[iHandle].ftDCB->MPWM_Sub1[n] = Power;
			break;
		case 2:
			if(OnOff) rbI[iHandle].ftDCB->M_Sub2 |=  EMaske[n];
			else      rbI[iHandle].ftDCB->M_Sub2 &= ~EMaske[n];
			rbI[iHandle].ftDCB->MPWM_Sub2[n] = Power;
			break;
		case 3:
			if(OnOff) rbI[iHandle].ftDCB->M_Sub3 |=  EMaske[n];
			else      rbI[iHandle].ftDCB->M_Sub3 &= ~EMaske[n];
			rbI[iHandle].ftDCB->MPWM_Sub3[n] = Power;
			break;
		default: return rbFehler;
	}
	return 0;
}

int __stdcall rbSetLamp(int iHandle, int LampNr, int OnOff) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	return rbSetLampEx(iHandle, LampNr, OnOff, 7);
}

// rbRobMotor -----------------------------------------------------------------
//
// Starten eines M-Ausganges im RobMode
//
// ------------------------------------------------------------------------------

int __stdcall rbRobMotor(int iHandle, int MotNr, int Dir, int Speed, int ICount) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	MotNr--;	
	rbI[iHandle].mModeStatus   &= ~MMaske[MotNr];
	rbI[iHandle].mModeStatus   |= 1 << (MotNr * 2);
	rbI[iHandle].iCounter[MotNr*2+1] = ICount;
	return SetMotorAll(iHandle, MotNr >> 2, MotNr % 4, Dir, Speed);
}

// rbRobMotors ----------------------------------------------------------------
//
// Setzen des vollstndigen Status der M-Ausgnge
//        die zugehrenden Counter mssen separat gesetzt werden
//
// ----------------------------------------------------------------------------

int __stdcall rbRobMotors(int iHandle, int MotorStatus, 
													int SpeedStatus, int SpeedStatus16, int ModeStatus) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	rbI[iHandle].ftDCB->M_Main =	MotorStatus & 0x000000FF;
	rbI[iHandle].ftDCB->M_Sub1 = (MotorStatus & 0x0000FF00) >> 8;
	rbI[iHandle].ftDCB->M_Sub2 = (MotorStatus & 0x00FF0000) >> 16;
	rbI[iHandle].ftDCB->M_Sub3 = (MotorStatus & 0xFF000000) >> 24;
	int i2, i21;
	char Speed;
	for(int i = 0; i < 4; i++) {
		i2  = i*2;
		i21 = i2+1;
		Speed = (SpeedStatus   & (0xF << (i*4))) >> (i*4);
		rbI[iHandle].ftDCB->MPWM_Main[i2]  =  Speed;
		rbI[iHandle].ftDCB->MPWM_Main[i21] =  Speed;
		Speed = (SpeedStatus   & (0xF << (i*4+16))) >> (i*4+16);
		rbI[iHandle].ftDCB->MPWM_Sub1[i2]  = Speed;
		rbI[iHandle].ftDCB->MPWM_Sub1[i21] = Speed;
		Speed = (SpeedStatus16 & (0xF << (i*4))) >> (i*4);
		rbI[iHandle].ftDCB->MPWM_Sub2[i2]  = Speed;
		rbI[iHandle].ftDCB->MPWM_Sub2[i21] = Speed;
		Speed = (SpeedStatus16 & (0xF << (i*4+16))) >> (i*4+16);
		rbI[iHandle].ftDCB->MPWM_Sub3[i2]  = Speed;
		rbI[iHandle].ftDCB->MPWM_Sub3[i21] = Speed;
	}
	rbI[iHandle].mModeStatus = ModeStatus;
	return 0;
}

// rb...Counter ---------------------------------------------------------------
// 
// Counter Befehle : rbGetCounter, rbSetCounter, rbClearCounters
//
// ----------------------------------------------------------------------------

int __stdcall rbGetCounter(int iHandle, int CounterNr) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	return rbI[iHandle].iCounter[CounterNr-1];
}

int __stdcall rbSetCounter(int iHandle, int CounterNr, int Value) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	rbI[iHandle].iCounter[CounterNr-1] = Value;
	return 0;
}

int __stdcall rbClearCounters(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	for(int i = 0; i < 8; i++) rbI[iHandle].iCounter[i] = 0;
	return 0;
}

// RF-Messages -------------------------------------------------------------------------
//
// Lschen Messagebuffer, Abholen Message, Senden Message
//
// -------------------------------------------------------------------------------------

int __stdcall rbClearMessagesIn(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	rfAnz = 0;
	rfIn  = 0;
	rfOut = 0;
	return 0;
}

int __stdcall rbClearMessagesOut(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(ClearFtMessageBuffer(rbI[iHandle].ftHandle)) return rbFehler;
	return 0;
}

int __stdcall rbGetMessage(int iHandle, MessageData* inN) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(rfAnz > 0) {
		inN->HwId    = rfMsg[rfOut].W.ucHwId;
		inN->SubId   = rfMsg[rfOut].W.ucSubId;
		inN->MsgId   = rfMsg[rfOut].W.uiMsgId;
		inN->Msg     = rfMsg[rfOut].W.uiMsg;
		rfOut = rfOut < rfMax ? rfOut+1 : 0;
		rfAnz--;
		return 0;
	}
	else return rbFehler;
}

int __stdcall rbIsMessage(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	return rfAnz;
}

int __stdcall rbSendMessage(int iHandle, MessageData* outN) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	SMESSAGE sm;
	sm.W.ucHwId  = MSG_HWID_RF;
	sm.W.ucSubId = outN->SubId;
	sm.W.uiMsg   = outN->Msg;
	sm.W.uiMsgId = outN->MsgId;
	if(SendFtMessage(rbI[iHandle].ftHandle, 
		MSG_HWID_RF, sm.L.ucSubId, sm.L.dw, 55, MSG_SEND_NORMAL)) return rbFehler;
	return 0;
}

int __stdcall rbSendMessageEx(int iHandle, MessageData* outN, int Spez) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	SMESSAGE sm;
	sm.W.ucHwId  = MSG_HWID_RF;
	sm.W.ucSubId = outN->SubId;
	sm.W.uiMsg   = outN->Msg;
	sm.W.uiMsgId = outN->MsgId;
	if(SendFtMessage(rbI[iHandle].ftHandle, 
		MSG_HWID_RF, sm.L.ucSubId, sm.L.dw, 55, Spez)) return rbFehler;
	return 0;
}

// rbDevice -----------------------------------------------------------------
//
// Auskunftsfunktionen : rbGetActDeviceType, SerialNr, FirmwareNr
//                                           Name,     Firmware
//
// ---------------------------------------------------------------------------
int __stdcall rbGetActDeviceType(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	return GetFtDeviceTyp(rbI[iHandle].ftHandle);
}
int __stdcall rbGetActDeviceSerialNr(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	return GetFtSerialNr(rbI[iHandle].ftHandle);
}
int __stdcall rbGetActDeviceFirmwareNr(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	return GetFtFirmware(rbI[iHandle].ftHandle);
}
LPCSTR __stdcall rbGetActDeviceFirmware(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return NULL;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return NULL;
	return GetFtFirmwareStrg(rbI[iHandle].ftHandle);
}
LPCSTR __stdcall rbGetActDeviceName(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return NULL;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return NULL;
	return GetFtShortNameStrg(rbI[iHandle].ftHandle);
}


// DllMain -------------------------------------------------------------------
//
// Hauptschleife bei Start und Ende
// Schlieen aller RoboInstanzen, wenn nicht bereits geschehen
// Schlieen FtLib Arbeitsbereich
//
// ---------------------------------------------------------------------------

BOOL WINAPI DllMain(HINSTANCE hDLL, DWORD dwReason, LPVOID) {
	switch(dwReason) {
		case DLL_PROCESS_DETACH:
			for(int i = 0; i < 8; i++) rbCloseInterface(i);
			CloseFtLib();
			break;
	}
	return TRUE;
}